@@ -47,6 +47,29 @@ class ClientComm(CommBase.CommBase):
                                          direction='send', no_suffix=True,
                                          address=self.ocomm.address)
 
+    def get_status_message(self, nindent=0, **kwargs):
+        r"""Return lines composing a status message.
+        
+        Args:
+            nindent (int, optional): Number of tabs that should
+                be used to indent each line. Defaults to 0.
+            *kwargs: Additional arguments are passed to the
+                parent class's method.
+                
+        Returns:
+            tuple(list, prefix): Lines composing the status message and the
+                prefix string used for the last message.
+
+        """
+        lines, prefix = super(ClientComm, self).get_status_message(
+            nindent=nindent, **kwargs)
+        lines.append('%s%-15s:' % (prefix, 'request comm'))
+        lines += self.ocomm.get_status_message(nindent=(nindent + 1))[0]
+        lines.append('%s%-15s:' % (prefix, 'response comms'))
+        for x in self.icomm.values():
+            lines += x.get_status_message(nindent=(nindent + 1))[0]
+        return lines, prefix
+    
     @classmethod
     def is_installed(cls, language=None):
         r"""Determine if the necessary libraries are installed for this
