@@ -5,11 +5,15 @@ logger = logging.getLogger(__name__)
 try:
     import pika
     _rmq_installed = True
+    _pika_version_maj = int(float(pika.__version__.split('.')[0]))
+    if _pika_version_maj >= 1:  # pragma: debug
+        raise ImportError("pika version 1.0 not yet supported.")
 except ImportError:
     logger.debug("Could not import pika. "
                  + "RabbitMQ support will be disabled.")
     pika = None
     _rmq_installed = False
+    _pika_version_maj = 0
 
 
 _rmq_param_sep = '_RMQPARAM_'
@@ -58,7 +62,9 @@ def check_rmq_server(url=None, **kwargs):
         if not connection.is_open:  # pragma: debug
             raise BaseException("Connection was not openned.")
         connection.close()
-    except BaseException:  # pragma: debug
+    except BaseException as e:  # pragma: debug
+        print("Error when attempting to connect to the RabbitMQ server: "
+              + str(e))
         out = False
     logging.getLogger("pika").propagate = True
     return out
