@@ -1,14 +1,21 @@
 import os
 import tempfile
 import uuid
-import zmq
 import logging
 from yggdrasil import tools
 from yggdrasil import multitasking
 from yggdrasil.communication import CommBase, AsyncComm
-
-
 logger = logging.getLogger(__name__)
+try:
+    import zmq
+    _zmq_installed = True
+except ImportError:  # pragma: debug
+    logger.debug("Could not import pyzmq. "
+                 + "ZMQ support will be disabled.")
+    zmq = None
+    _zmq_installed = False
+
+
 _socket_type_pairs = [('PUSH', 'PULL'),
                       ('PUB', 'SUB'),
                       ('REP', 'REQ'),
@@ -23,7 +30,10 @@ _default_protocol = 'tcp'
 _wait_send_t = 0  # 0.0001
 _reply_msg = b'YGG_REPLY'
 _purge_msg = b'YGG_PURGE'
-_global_context = zmq.Context.instance()
+if _zmq_installed:
+    _global_context = zmq.Context.instance()
+else:  # pragma: debug
+    _global_context = None
 
 
 def get_ipc_host():
@@ -212,9 +222,16 @@ class ZMQProxy(CommBase.CommServer):
         # Bind backend
         self.srv_socket = zmq_context.socket(zmq.DEALER)
         self.srv_socket.setsockopt(zmq.LINGER, 0)
-        self.srv_address = bind_socket(self.srv_socket, srv_address,
-                                       nretry=nretry,
-                                       retry_timeout=retry_timeout)
+        try:
+            self.srv_address = bind_socket(self.srv_socket, srv_address,
+                                           nretry=nretry,
+                                           retry_timeout=retry_timeout)
+        except zmq.ZMQError:  # pragma: debug
+            self.cli_socket.close()
+            self.cli_socket = None
+            CommBase.unregister_comm(
+                'ZMQComm', 'ROUTER_server_' + self.cli_address)
+            raise
         CommBase.register_comm('ZMQComm', 'DEALER_server_' + self.srv_address,
                                self.srv_socket)
         # Set up poller
