@@ -226,7 +226,7 @@ class MakeCompiler(BuildToolBase):
         compiler = language_driver.get_tool('compiler', toolname=language_compiler)
         compile_flags = language_driver.get_compiler_flags(
             for_model=True, skip_defaults=True, dont_skip_env_defaults=True,
-            logging_level=logging_level, dont_link=True, toolnane=language_compiler)
+            logging_level=logging_level, dont_link=True, toolname=language_compiler)
         linker = language_driver.get_tool('linker', toolname=language_compiler)
         linker_flags = language_driver.get_linker_flags(
             for_model=True, skip_defaults=True, dont_skip_env_defaults=True,
