@@ -18,6 +18,10 @@ extern "C" {
 #define ygg_free free_comm
 #define yggComm init_comm
 
+// Forward declaration of server interface to allow replacemnt
+static inline
+comm_t* yggRpcServerType_global(const char *name, dtype_t *inType, dtype_t *outType);
+
 //==============================================================================
 /*!
   Basic IO 
@@ -52,6 +56,19 @@ extern "C" {
  */
 static inline
 yggOutput_t yggOutputType(const char *name, dtype_t *datatype) {
+  const char* YGG_SERVER_OUTPUT = getenv("YGG_SERVER_OUTPUT");
+  if (YGG_SERVER_OUTPUT) {
+    const char* YGG_MODEL_NAME = getenv("YGG_MODEL_NAME");
+    if (strcmp(name, YGG_SERVER_OUTPUT) == 0) {
+      return yggRpcServerType_global(YGG_MODEL_NAME, NULL, datatype);
+    } else {
+      char alt_name[100];
+      sprintf(alt_name, "%s:%s", YGG_MODEL_NAME, name);
+      if (strcmp(alt_name, YGG_SERVER_OUTPUT) == 0) {
+	return yggRpcServerType_global(YGG_MODEL_NAME, NULL, datatype);
+      }
+    }
+  }
   return init_comm(name, "send", _default_comm, datatype);
 };
 
@@ -66,6 +83,19 @@ yggOutput_t yggOutputType(const char *name, dtype_t *datatype) {
  */
 static inline
 yggInput_t yggInputType(const char *name, dtype_t *datatype) {
+  const char* YGG_SERVER_INPUT = getenv("YGG_SERVER_INPUT");
+  if (YGG_SERVER_INPUT) {
+    const char* YGG_MODEL_NAME = getenv("YGG_MODEL_NAME");
+    if (strcmp(name, YGG_SERVER_INPUT) == 0) {
+      return yggRpcServerType_global(YGG_MODEL_NAME, datatype, NULL);
+    } else {
+      char alt_name[100];
+      sprintf(alt_name, "%s:%s", YGG_MODEL_NAME, name);
+      if (strcmp(alt_name, YGG_SERVER_INPUT) == 0) {
+	return yggRpcServerType_global(YGG_MODEL_NAME, datatype, NULL);
+      }
+    }
+  }
   return init_comm(name, "recv", _default_comm, datatype);
 };
   
@@ -81,7 +111,15 @@ yggInput_t yggInputType(const char *name, dtype_t *datatype) {
  */
 static inline
 yggOutput_t yggOutputFmt(const char *name, const char *fmtString){
-  return init_comm_format(name, "send", _default_comm, fmtString, 0);
+  dtype_t* datatype = formatstr2datatype(fmtString, 0);
+  comm_t* out = yggOutputType(name, datatype);
+  if ((fmtString != NULL) && (datatype == NULL)) {
+    ygglog_error("yggOutputFmt: Failed to create tyep from format_str.");
+    if (out != NULL) {
+      out->valid = 0;
+    }
+  }
+  return out;
 };
 
 /*!
@@ -95,7 +133,15 @@ yggOutput_t yggOutputFmt(const char *name, const char *fmtString){
  */
 static inline
 yggInput_t yggInputFmt(const char *name, const char *fmtString){
-  return init_comm_format(name, "recv", _default_comm, fmtString, 0);
+  dtype_t* datatype = formatstr2datatype(fmtString, 0);
+  comm_t* out = yggInputType(name, datatype);
+  if ((fmtString != NULL) && (datatype == NULL)) {
+    ygglog_error("yggInputFmt: Failed to create tyep from format_str.");
+    if (out != NULL) {
+      out->valid = 0;
+    }
+  }
+  return out;
 };
 
 /*!
@@ -107,7 +153,7 @@ yggInput_t yggInputFmt(const char *name, const char *fmtString){
  */
 static inline
 yggOutput_t yggOutput(const char *name) {
-  yggOutput_t ret = yggOutputFmt(name, NULL);
+  yggOutput_t ret = yggOutputType(name, NULL);
   return ret;
 };
 
@@ -120,7 +166,7 @@ yggOutput_t yggOutput(const char *name) {
  */
 static inline
 yggInput_t yggInput(const char *name){
-  yggInput_t ret = yggInputFmt(name, NULL);
+  yggInput_t ret = yggInputType(name, NULL);
   return ret;
 };
 
@@ -378,6 +424,90 @@ comm_t* yggRpcServer(const char *name, const char *inFormat, const char *outForm
   return init_comm_format(name, inFormat, SERVER_COMM, outFormat, 0);
 };
 
+/*!
+  @brief Constructor for client side RPC structure w/ explicit type info.
+  Creates an instance of yggRpc_t with provided information.  
+  @param[in] name constant character pointer to name for queues.
+  @param[in] outType Pointer to a dtype_t data structure containing type info
+  for data that will be sent by the client.
+  @param[in] inType Pointer to a dtype_t data structure containing type info
+  for data that will be received by the client.
+  @return yggRpc_t structure with provided info.
+ */
+static inline
+comm_t* yggRpcClientType(const char *name, dtype_t *outType, dtype_t *inType) {
+  // Allow for any type if provided is NULL
+  if (outType == NULL) {
+    outType = create_dtype_empty(true);
+  }
+  if (inType == NULL) {
+    inType = create_dtype_empty(true);
+  }
+  comm_t* out = init_comm(name, "%s", CLIENT_COMM, inType);
+  comm_t* handle = (comm_t*)(out->handle);
+  destroy_dtype(&(handle->datatype));
+  handle->datatype = outType;
+  return out;
+};
+
+/*!
+  @brief Constructor for server side RPC structure w/ explicit type info.
+  Creates an instance of yggRpc_t with provided information.  
+  @param[in] name constant character pointer to name for queues.
+  @param[in] inType Pointer to a dtype_t data structure containing type info
+  for data that will be received by the server.
+  @param[in] outType Pointer to a dtype_t data structure containing type info
+  for data that will be sent by the server.
+  @return yggRpc_t structure with provided info.
+ */
+static inline
+comm_t* yggRpcServerType(const char *name, dtype_t *inType, dtype_t *outType) {
+  // Allow for any type if provided is NULL
+  if (inType == NULL) {
+    inType = create_dtype_empty(true);
+  }
+  if (outType == NULL) {
+    outType = create_dtype_empty(true);
+  }
+  comm_t* out = init_comm(name, "%s", SERVER_COMM, outType);
+  comm_t* handle = (comm_t*)(out->handle);
+  destroy_dtype(&(handle->datatype));
+  handle->datatype = inType;
+  return out;
+};
+
+/*!
+  @brief Constructor for server side RPC structure w/ explicit type info.
+  Creates an instance of yggRpc_t with provided information after first
+  checking for a pre-existing global comm of the same name. If one doesn't
+  exist, one is created.
+  @param[in] name constant character pointer to name for queues.
+  @param[in] inType Pointer to a dtype_t data structure containing type info
+  for data that will be received by the server.
+  @param[in] outType Pointer to a dtype_t data structure containing type info
+  for data that will be sent by the server.
+  @return yggRpc_t structure with provided info.
+ */
+static inline
+comm_t* yggRpcServerType_global(const char *name, dtype_t *inType, dtype_t *outType) {
+  comm_t* out = NULL;
+  WITH_GLOBAL_SCOPE(out = get_global_scope_comm(name));
+  if (out == NULL) {
+    WITH_GLOBAL_SCOPE(out = yggRpcServerType(name, inType, outType));
+    return out;
+  }
+  if (inType != NULL) {
+    comm_t* handle = (comm_t*)(out->handle);
+    destroy_dtype(&(handle->datatype));
+    handle->datatype = inType;
+  }
+  if (outType != NULL) {
+    destroy_dtype(&(out->datatype));
+    out->datatype = outType;
+  }
+  return out;
+};
+
 /*!
   @brief Constructor for client side timestep synchronization calls.
   Creates an instance of comm_t with provided information.  
@@ -392,9 +522,7 @@ comm_t* yggTimesync(const char *name, const char *t_units) {
   dtypes_out[0] = create_dtype_scalar("float", 64, t_units, false);
   dtypes_out[1] = create_dtype_json_object(0, NULL, NULL, true);
   dtype_t* dtype_out = create_dtype_json_array(2, dtypes_out, false);
-  dtype_t* dtypes_in[1];
-  dtypes_in[0] = create_dtype_json_object(0, NULL, NULL, true);
-  dtype_t* dtype_in = create_dtype_json_array(1, dtypes_in, false);
+  dtype_t* dtype_in = create_dtype_json_object(0, NULL, NULL, true);
   comm_t* out = init_comm(name, "%s", CLIENT_COMM, dtype_in);
   comm_t* handle = (comm_t*)(out->handle);
   destroy_dtype(&(handle->datatype));
@@ -488,7 +616,12 @@ int vrpcCallBase(yggRpc_t rpc, const int allow_realloc,
   
   // pack the args and call
   comm_t *send_comm = (comm_t*)(rpc->handle);
-  size_t send_nargs = nargs_exp_dtype(send_comm->datatype);
+  size_t send_nargs;
+  if (is_empty_dtype(send_comm->datatype)) {
+    send_nargs = 1;
+  } else {
+    send_nargs = nargs_exp_dtype(send_comm->datatype);
+  }
   sret = vcommSend(rpc, send_nargs, ap);
   if (sret < 0) {
     ygglog_error("vrpcCall: vcommSend error: ret %d: %s", sret, strerror(errno));
@@ -497,11 +630,12 @@ int vrpcCallBase(yggRpc_t rpc, const int allow_realloc,
 
   // Advance through sent arguments
   ygglog_debug("vrpcCall: Used %d arguments in send", sret);
-  int i;
-  for (i = 0; i < sret; i++) {
-    va_list_t_skip(&op, sizeof(void*));
+  if (sret > 0) {
+    if (skip_va_elements(send_comm->datatype, &nargs, &op)) {
+      ygglog_error("vrpcCall: Error skipping send arguments.");
+      return -1;
+    }
   }
-  nargs = nargs - sret;
 
   // unpack the messages into the remaining variable arguments
   rret = vcommRecv(rpc, allow_realloc, nargs, op);
