@@ -42,9 +42,12 @@ typedef struct comm_t {
   void *reply; //!< Reply information.
   int is_file; //!< Flag specifying if the comm connects directly to a file.
   int is_work_comm; //!< Flag specifying if comm is a temporary work comm.
+  int is_global; //!< Flag specifying if the comm is global.
+  int thread_id; //!< ID for the thread that created the comm.
 } comm_t;
 
 
+static inline
 void display_other(comm_t *x) {
   if (x->other != NULL) {
     comm_t* other = (comm_t*)(x->other);
@@ -60,6 +63,7 @@ void display_other(comm_t *x) {
 */
 static inline
 int free_comm_base(comm_t *x) {
+  ygglog_debug("free_comm_base: Started");
   if (x == NULL)
     return 0;
   if (x->last_send != NULL) {
@@ -85,6 +89,7 @@ int free_comm_base(comm_t *x) {
   x->valid = 0;
   x->name[0] = '\0';
   x->index_in_register = -1;
+  ygglog_debug("free_comm_base: Finished");
   return 0;
 };
 
@@ -115,6 +120,8 @@ comm_t empty_comm_base() {
   ret.reply = NULL;
   ret.is_file = 0;
   ret.is_work_comm = 0;
+  ret.is_global = 0;
+  ret.thread_id = 0;
   return ret;
 };
 
@@ -180,6 +187,7 @@ comm_t* new_comm_base(char *address, const char *direction,
   ret->sent_eof[0] = 0;
   ret->recv_eof[0] = 0;
   ret->used[0] = 0;
+  ret->thread_id = get_thread_id();
   return ret;
 };
 
@@ -220,6 +228,18 @@ comm_t* init_comm_base(const char *name, const char *direction,
 	address = getenv(full_name);
       }
     }
+    if (address == NULL) {
+      char temp_name[COMM_NAME_SIZE] = "";
+      size_t i;
+      for (i = 0; i < strlen(full_name); i++) {
+	if (full_name[i] == ':') {
+	  strcat(temp_name, "__COLON__");
+	} else {
+	  strncat(temp_name, full_name + i, 1);
+	}
+      }
+      address = getenv(temp_name);
+    }
     ygglog_debug("init_comm_base: model_name = %s, full_name = %s, address = %s",
 		 model_name, full_name, address);
   }
