@@ -21,6 +21,58 @@ extern "C" {
 static void **vcomms2clean = NULL;
 static size_t ncomms2clean = 0;
 static size_t clean_registered = 0;
+static size_t clean_called = 0;
+
+/*! @brief Memory to keep track of global scope comms. */
+static size_t global_scope_comm = 0;
+#ifdef _OPENMP
+#pragma omp threadprivate(global_scope_comm)
+#endif
+#define WITH_GLOBAL_SCOPE(COMM) global_scope_comm = 1; COMM; global_scope_comm = 0
+
+
+/*!
+  @brief Retrieve a registered global comm if it exists.
+  @param[in] const char* name Name that comm might be registered under.
+  @returns comm_t* Pointer to registered comm. NULL if one does not exist
+  with the specified name.
+ */
+static
+comm_t* get_global_scope_comm(const char *name) {
+  comm_t* out = NULL;
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
+#endif
+  if (global_scope_comm) {
+    size_t i;
+    comm_t* icomm = NULL;
+    int current_thread = get_thread_id();
+    printf("CURRENT THREAD: %d\n", current_thread);
+    for (i = 0; i < ncomms2clean; i++) {
+      if (vcomms2clean[i] != NULL) {
+	icomm = (comm_t*)(vcomms2clean[i]);
+	if ((strcmp(icomm->name, name) == 0) && (icomm->thread_id == current_thread)) {
+	  out = icomm;
+	  break;
+	} else {
+	  const char* YGG_MODEL_NAME = getenv("YGG_MODEL_NAME");
+	  char alt_name[100];
+	  sprintf(alt_name, "%s:%s", YGG_MODEL_NAME, name);
+	  if ((strcmp(icomm->name, alt_name) == 0) && (icomm->thread_id == current_thread)) {
+	    out = icomm;
+	    break;
+	  }
+	}
+      }
+    }
+  }
+#ifdef _OPENMP
+  }
+#endif
+  return out;
+};
+
 
 // Forward declaration of eof
 static
@@ -50,6 +102,10 @@ static
 int free_comm_type(comm_t *x) {
   comm_type t = x->type;
   int ret = 1;
+  if (x->thread_id != get_thread_id()) {
+    ygglog_error("free_comm_type: Thread is attempting to use a comm it did not initialize");
+    return ret;
+  }
   if (t == IPC_COMM)
     ret = free_ipc_comm(x);
   else if (t == ZMQ_COMM)
@@ -78,6 +134,10 @@ int free_comm(comm_t *x) {
   int ret = 0;
   if (x == NULL)
     return ret;
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
+#endif
   ygglog_debug("free_comm(%s)", x->name);
   // Send EOF for output comms and then wait for messages to be recv'd
   if ((is_send(x->direction)) && (x->valid)) {
@@ -102,6 +162,10 @@ int free_comm(comm_t *x) {
       vcomms2clean[idx] = NULL;
     }
   }
+  ygglog_debug("free_comm: Finished");
+#ifdef _OPENMP
+  }
+#endif
   return ret;
 };
 
@@ -111,29 +175,68 @@ int free_comm(comm_t *x) {
 static
 void clean_comms(void) {
   size_t i;
-  ygglog_debug("atexit begin");
-  for (i = 0; i < ncomms2clean; i++) {
-    if (vcomms2clean[i] != NULL) {
-      free_comm((comm_t*)(vcomms2clean[i]));
-    }
-  }
-  if (vcomms2clean != NULL) {
-    free(vcomms2clean);
-    vcomms2clean = NULL;
-  }
-  ncomms2clean = 0;
-#if defined(ZMQINSTALLED)
-  // #if defined(_MSC_VER) && defined(ZMQINSTALLED)
-  zsys_shutdown();
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
 #endif
-  if (Py_IsInitialized()) {
-    Py_Finalize();
+  if (!(clean_called)) {
+    ygglog_debug("atexit begin");
+    for (i = 0; i < ncomms2clean; i++) {
+      if (vcomms2clean[i] != NULL) {
+	free_comm((comm_t*)(vcomms2clean[i]));
+      }
+    }
+    if (vcomms2clean != NULL) {
+      free(vcomms2clean);
+      vcomms2clean = NULL;
+    }
+    ncomms2clean = 0;
+#if defined(ZMQINSTALLED)
+    // #if defined(_MSC_VER) && defined(ZMQINSTALLED)
+    ygg_zsys_shutdown();
+#endif
+    if (Py_IsInitialized()) {
+      Py_Finalize();
+    }
+  /* printf(""); */
+    clean_called = 1;
   }
+#ifdef _OPENMP
+  }
+#endif
   ygglog_debug("atexit done");
   if (_ygg_error_flag != 0) {
     _exit(_ygg_error_flag);
   }
-  /* printf(""); */
+};
+
+/*!
+  @brief Initialize yggdrasil in a thread-safe way
+ */
+static inline
+int ygg_init() {
+  int out = 0;
+#ifdef _OPENMP
+#pragma omp critical (init)
+  {
+#endif
+  ygglog_debug("ygg_init: clean_registered = %d", clean_registered);
+  if (clean_registered == 0) {
+#if defined(ZMQINSTALLED)
+    if (!(ygg_zsys_init())) {
+      out = -1;
+    }
+#endif
+    if (out == 0) {
+      ygglog_debug("ygg_init: Registering cleanup");
+      atexit(clean_comms);
+      clean_registered = 1;
+    }
+  }
+#ifdef _OPENMP
+  }
+#endif
+  return out;
 };
 
 /*!
@@ -144,22 +247,31 @@ void clean_comms(void) {
  */
 static
 int register_comm(comm_t *x) {
-  if (clean_registered == 0) {
-    atexit(clean_comms);
-    clean_registered = 1;
-  }
   if (x == NULL) {
     return 0;
   }
-  void **t_vcomms2clean = (void**)realloc(vcomms2clean, sizeof(void*)*(ncomms2clean + 1));
-  if (t_vcomms2clean == NULL) {
-    ygglog_error("register_comm(%s): Failed to realloc the comm list.", x->name);
-    return -1;
+  int error_flag = 0;
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
+#endif
+    if (ygg_init()) {
+      error_flag = 1;
+    } else {
+      void **t_vcomms2clean = (void**)realloc(vcomms2clean, sizeof(void*)*(ncomms2clean + 1));
+      if (t_vcomms2clean == NULL) {
+	ygglog_error("register_comm(%s): Failed to realloc the comm list.", x->name);
+	error_flag = -1;
+      } else {
+	vcomms2clean = t_vcomms2clean;
+	x->index_in_register = (int)ncomms2clean;
+	vcomms2clean[ncomms2clean++] = (void*)x;
+      }
+    }
+#ifdef _OPENMP
   }
-  vcomms2clean = t_vcomms2clean;
-  x->index_in_register = (int)ncomms2clean;
-  vcomms2clean[ncomms2clean++] = (void*)x;
-  return 0;
+#endif
+  return error_flag;
 };
 
 /*!
@@ -285,10 +397,14 @@ comm_t* init_comm(const char *name, const char *direction,
   SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX);
   _set_abort_behavior(0,_WRITE_ABORT_MSG);
 #endif
+  comm_t *ret = get_global_scope_comm(name);
+  if (ret != NULL) {
+    return ret;
+  }
   if ((datatype == NULL) && (strcmp(direction, "send") == 0)) {
     datatype = create_dtype_scalar("bytes", 0, "", false);
   }
-  comm_t *ret = init_comm_base(name, direction, t, datatype);
+  ret = init_comm_base(name, direction, t, datatype);
   if (ret == NULL) {
     ygglog_error("init_comm(%s): Could not initialize base.", name);
     return ret;
@@ -304,10 +420,30 @@ comm_t* init_comm(const char *name, const char *direction,
       ret->valid = 0;
     }
   }
+  if (global_scope_comm) {
+    ret->is_global = 1;
+    ygglog_debug("init_comm(%s): Global comm!", name);
+  }
   ygglog_debug("init_comm(%s): Initialized comm.", name);
   return ret;
 };
 
+
+/*!
+  @brief Convert a format string to a datatype.
+  @param[in] format_str char* Format string.
+  @param[in] as_array int If 1, inputs/outputs are processed as arrays.
+  @returns dtype_t* Pointer to datatype structure.
+ */
+static
+dtype_t* formatstr2datatype(const char *format_str, const int as_array) {
+  dtype_t* datatype = NULL;
+  if (format_str != NULL) {
+    datatype = create_dtype_format(format_str, as_array, false);
+  }
+  return datatype;
+};
+
 /*!
   @brief Initialize a generic communicator using a format string to determine
   the type.
@@ -326,16 +462,7 @@ static
 comm_t* init_comm_format(const char *name, const char *direction,
 			 const comm_type t, const char *format_str,
 			 const int as_array) {
-  dtype_t* datatype = NULL;
-  if (format_str == NULL) {
-    if (strcmp(direction, "recv") == 0) {
-      datatype = NULL; // It will be set on receiving a message
-    } else {
-      datatype = NULL;
-    }
-  } else {
-    datatype = create_dtype_format(format_str, as_array, false);
-  }
+  dtype_t* datatype = formatstr2datatype(format_str, as_array);
   comm_t* out = init_comm(name, direction, t, datatype);
   if ((format_str != NULL) && (datatype == NULL)) {
     ygglog_error("init_comm_format: Failed to create type from format_str.");
@@ -395,6 +522,10 @@ int comm_send_single(const comm_t *x, const char *data, const size_t len) {
     ygglog_error("comm_send_single: Invalid comm");
     return ret;
   }
+  if (x->thread_id != get_thread_id()) {
+    ygglog_error("comm_send_single: Thread is attempting to use a comm it did not initialize");
+    return ret;
+  }
   comm_type t = x->type;
   if (t == IPC_COMM)
     ret = ipc_comm_send(x, data, len);
@@ -738,6 +869,10 @@ int comm_recv_single(comm_t *x, char **data, const size_t len,
     ygglog_error("comm_recv_single: Invalid comm");
     return ret;
   }
+  if (x->thread_id != get_thread_id()) {
+    ygglog_error("comm_recv_single: Thread is attempting to use a comm it did not initialize");
+    return ret;
+  }
   comm_type t = x->type;
   if (t == IPC_COMM)
     ret = ipc_comm_recv(x, data, len, allow_realloc);
