@@ -132,6 +132,15 @@ public:
   size_t nargs_exp() const override {
     return 1;
   }
+  /*!
+    @brief Skip arguments that make of this type.
+    @param[in, out] nargs Pointer to number of arguments in ap.
+    @param[in, out] ap va_list_t Variable argument list.
+   */
+  void skip_va_elements_core(size_t *nargs, va_list_t *ap) const override {
+    va_arg(ap->va, dtype_t*);
+    (*nargs)--;
+  }
   /*!
     @brief Convert a Python representation to a C representation.
     @param[in] pyobj PyObject* Pointer to Python object.
