@@ -42,7 +42,10 @@ class PandasSerialize(AsciiTableSerialize):
     @property
     def empty_msg(self):
         r"""obj: Object indicating empty message."""
-        return pandas.DataFrame(np.zeros(0, self.numpy_dtype))
+        if self.numpy_dtype:
+            return pandas.DataFrame(np.zeros(0, self.numpy_dtype))
+        else:
+            return pandas.DataFrame(columns=self.get_field_names())
 
     def get_field_names(self, *args, **kwargs):
         r"""Get the field names for an array of fields.
@@ -154,7 +157,7 @@ class PandasSerialize(AsciiTableSerialize):
                      # Not in pandas <0.24
                      # line_terminator=self.newline.decode("utf-8"),
                      sep=self.delimiter.decode("utf-8"),
-                     mode='wb', encoding='utf8',
+                     mode='w', encoding='utf8',
                      header=(not self.dont_write_header))
         if self.write_header_once:
             self.dont_write_header = True
