@@ -1,3 +1,4 @@
+import os
 import copy
 import pprint
 import numpy as np
@@ -213,16 +214,19 @@ class SerializeBase(tools.YggClass):
             assert(table_string_type in ['bytes', 'unicode', 'string'])
             if table_string_type == 'string':
                 table_string_type = 'unicode'
+            int_type = np.int32
+            if np.dtype(int) == int_type:  # pragma: windows
+                int_type = int
             if table_string_type == 'bytes':
                 np_dtype_str = 'S'
-                rows = [(b'one', np.int32(1), 1.0),
-                        (b'two', np.int32(2), 2.0),
-                        (b'three', np.int32(3), 3.0)]
+                rows = [(b'one', int_type(1), 1.0),
+                        (b'two', int_type(2), 2.0),
+                        (b'three', int_type(3), 3.0)]
             else:
                 np_dtype_str = 'U'
-                rows = [('one', np.int32(1), 1.0),
-                        ('two', np.int32(2), 2.0),
-                        ('three', np.int32(3), 3.0)]
+                rows = [('one', int_type(1), 1.0),
+                        ('two', int_type(2), 2.0),
+                        ('three', int_type(3), 3.0)]
             out = {'kwargs': {}, 'empty': [], 'dtype': None,
                    'extra_kwargs': {},
                    'typedef': {'type': 'array',
@@ -712,8 +716,9 @@ class SerializeBase(tools.YggClass):
                         metadata['metadata'] = {
                             'datatype': self.datatype.encode_type(
                                 args, typedef=self.typedef)}
-        if ((self.initialized
-             and (not tools.check_environ_bool('YGG_VALIDATE_ALL_MESSAGES')))):
+        validate_msgs = os.environ.get('YGG_VALIDATE_MESSAGES', 'first').lower()
+        if (((self.initialized and (validate_msgs == 'first'))
+             or (validate_msgs in ['false', '0']))):
             metadata.setdefault('dont_check', True)
         out = self.encoded_datatype.serialize(data, **metadata)
         return out
@@ -736,8 +741,9 @@ class SerializeBase(tools.YggClass):
         if (((self.func_deserialize is not None)
              and (self.encoded_typedef['type'] == 'bytes'))):
             kwargs['dont_decode'] = True
-        if ((self.initialized
-             and (not tools.check_environ_bool('YGG_VALIDATE_ALL_MESSAGES')))):
+        validate_msgs = os.environ.get('YGG_VALIDATE_MESSAGES', 'first').lower()
+        if (((self.initialized and (validate_msgs == 'first'))
+             or (validate_msgs in ['false', '0']))):
             kwargs.setdefault('dont_check', True)
         out, metadata = self.encoded_datatype.deserialize(msg, **kwargs)
         if (self.func_deserialize is not None):
