@@ -13,6 +13,8 @@ import psutil
 import copy
 import pprint
 import types
+import warnings
+import json
 from pandas.testing import assert_frame_equal
 from yggdrasil.config import ygg_cfg, cfg_logging
 from yggdrasil import tools, platform, units
@@ -41,7 +43,8 @@ script_list = [
     ('lpy', 'lpy_model.lpy'),
     ('r', 'r_model.R'),
     ('fortran', ['hellofunc.f90', 'fortran_model.f90']),
-    ('sbml', 'sbml_model.xml')]
+    ('sbml', 'sbml_model.xml'),
+    ('osr', 'osr_model.xml')]
 scripts = {}
 for k, v in script_list:
     if isinstance(v, list):
@@ -49,8 +52,9 @@ for k, v in script_list:
     else:
         scripts[k] = os.path.join(script_dir, v)
 # scripts = {k: os.path.join(script_dir, v) for k, v in script_list}
-if platform._is_win:  # pragma: windows
-    scripts['executable'] = ['timeout', '0']
+if platform._is_win and (not tools.in_powershell()):  # pragma: windows
+    # scripts['executable'] = ['timeout', '0']
+    scripts['executable'] = ['ping', '-n', '1', '127.0.0.1']
 else:
     scripts['executable'] = ['sleep', 0.1]
     
@@ -66,7 +70,8 @@ yaml_list = [
     ('error', 'error_model.yml'),
     ('lpy', 'lpy_model.yml'),
     ('fortran', 'fortran_model.yml'),
-    ('sbml', 'sbml_model.yml')]
+    ('sbml', 'sbml_model.yml'),
+    ('osr', 'osr_model.yml')]
 yamls = {k: os.path.join(yaml_dir, v) for k, v in yaml_list}
 
 # Makefile
@@ -82,25 +87,45 @@ enable_long_tests = tools.check_environ_bool("YGG_ENABLE_LONG_TESTS")
 skip_extra_examples = tools.check_environ_bool("YGG_SKIP_EXTRA_EXAMPLES")
 
 
-def check_enabled_languages(language):
+def check_enabled_languages(language, return_decorators=False):
     r"""Determine if the specified language is enabled by the value
     or values specified in YGG_TEST_LANGUAGE.
 
     Args:
         language (str): Language to check.
+        return_decorators (bool, optional): If True, a skip error is
+            not raised and a list of decorators is returned. Defaults
+            to False.
 
     Raises:
         unittest.SkipTest: If the specified language is not enabled.
 
     """
     enabled = os.environ.get('YGG_TEST_LANGUAGE', None)
+    decorators = []
     if enabled is not None:
-        enabled = [x.lower() for x in enabled.split(',')]
-        if ('c++' in enabled) or ('cpp' in enabled):
-            enabled += ['c++', 'cpp']
-        if language.lower() not in enabled:
-            raise unittest.SkipTest("Tests for language %s not enabled.",
-                                    language)
+        enabled = [x.lower() for x in json.loads(enabled)]
+        if ('c++' in enabled) or ('cpp' in enabled) or ('cxx' in enabled):
+            enabled += ['c++', 'cpp', 'cxx']
+        msg = "Tests for language %s not enabled." % language
+        cond = (language.lower() not in enabled)
+        if return_decorators:
+            decorators.append(unittest.skipIf(cond, msg))
+        elif cond:
+            raise unittest.SkipTest(msg)
+    disabled = os.environ.get('YGG_TEST_SKIP_LANGUAGE', None)
+    if disabled is not None:
+        disabled = [x.lower() for x in json.loads(disabled)]
+        if ('c++' in disabled) or ('cpp' in disabled):
+            disabled += ['c++', 'cpp']
+        msg = "Tests for language %s disabled." % language
+        cond = (language.lower() in disabled)
+        if return_decorators:
+            decorators.append(unittest.skipIf(cond, msg))
+        elif cond:  # pragma: debug
+            raise unittest.SkipTest(msg)
+    if return_decorators:
+        return decorators
 
 
 def requires_language(language, installed=True):
@@ -135,10 +160,7 @@ def requires_language(language, installed=True):
         elif installed is False:
             skips.append(unittest.skipIf(drv.is_installed(),
                                          "%s installed"))
-        skips.append(unittest.skipIf(
-            (test_language is not None)
-            and (test_language != drv.language.lower()),
-            "Test for language %s not enabled." % drv.language))
+        skips += check_enabled_languages(language, return_decorators=True)
         for s in skips:
             function = s(function)
         return function
@@ -146,6 +168,13 @@ def requires_language(language, installed=True):
     return wrapper
 
 
+def pprint_diff(x, y):  # pragma: no cover
+    r"""Get the diff between the pprint.pformat string for two objects."""
+    tools.print_encoded('\n'.join(difflib.ndiff(
+        pprint.pformat(x).splitlines(),
+        pprint.pformat(y).splitlines())))
+
+
 # Wrapped class to allow handling of arrays
 class WrappedTestCase(unittest.TestCase):  # pragma: no cover
     def __init__(self, *args, **kwargs):
@@ -419,7 +448,7 @@ def assert_warns(warning, *args, **kwargs):
     yield ut.assertWarns(warning, *args, **kwargs)
 
 
-def assert_equal(x, y):
+def assert_equal(x, y, dont_print_diff=False):
     r"""Assert that two messages are equivalent.
 
     Args:
@@ -430,7 +459,14 @@ def assert_equal(x, y):
         AssertionError: If the two messages are not equivalent.
 
     """
-    ut.assertEqual(x, y)
+    try:
+        with warnings.catch_warnings():
+            warnings.simplefilter(action='ignore', category=FutureWarning)
+            ut.assertEqual(x, y)
+    except AssertionError:  # pragma: debug
+        if (not dont_print_diff) and (type(x) == type(y)):
+            pprint_diff(x, y)
+        raise
 
 
 def assert_not_equal(x, y):
