@@ -28,6 +28,7 @@ def get_ureg():
         unyt._unit_lookup_table.inv_name_alternatives["acre"] = "ac"
         unyt._unit_lookup_table.inv_name_alternatives["are"] = "a"
         unyt._unit_lookup_table.inv_name_alternatives["hectare"] = "ha"
+        unyt._unit_lookup_table.inv_name_alternatives["days"] = "day"
     return _ureg_unyt
 
 
@@ -105,34 +106,81 @@ def convert_R_unit_string(r_str):
         str: Converted string.
 
     """
+    return convert_unit_string(r_str)
+
+
+def convert_unit_string(orig_str, replacements=None):
+    r"""Convert unit string to string that the Python package can
+    understand.
+
+    Args:
+        orig_str (str): Original units string to convert.
+        replacements (dict, optional): Mapping from unit to another.
+            Defaults to empty dict.
+
+    Returns:
+        str: Converted string.
+
+    """
+    if not orig_str.strip():
+        return ''
     out = []
-    replacements = {'h': 'hr'}
+    if replacements is None:
+        replacements = {'h': 'hr',
+                        'hrs': 'hr',
+                        'days': 'day',
+                        '100%': 'percent'}
     regex_mu = [tools.bytes2str(b'\xc2\xb5'),
-                tools.bytes2str(b'\xce\xbcs')]
-    regex = r'(?P<name>[A-Za-z%s]+)(?P<exp>-?[0-9]*)(?: |$)' % ''.join(regex_mu)
-    for x in re.finditer(regex, r_str):
-        xdict = x.groupdict()
-        if xdict['name'] in replacements:
-            xdict['name'] = replacements[xdict['name']]
-        if xdict['exp']:
-            out.append('({name}**{exp})'.format(**xdict))
+                tools.bytes2str(b'\xce\xbcs'),
+                tools.bytes2str(b'\xc2\xb0'),
+                r'(?:100\%)']
+    regex = (r'(?P<paren>\()?(?P<name>[A-Za-z%s]+)'
+             r'(?:(?:\*\*)?(?P<exp_paren>\()?(?P<exp>-?[0-9]+)'
+             r'(?(exp_paren)\)))?'
+             r'(?(paren)\)|)(?P<op> |(?:\*)|(?:\/))?' % ''.join(regex_mu))
+    out = ''
+    if re.fullmatch(r'(?:%s)+' % regex, orig_str.strip()):
+        for x in re.finditer(regex, orig_str.strip()):
+            xdict = x.groupdict()
+            if xdict['name'] in replacements:
+                xdict['name'] = replacements[xdict['name']]
+            if xdict['exp']:
+                out += '({name}**{exp})'.format(**xdict)
+            else:
+                out += xdict['name']
+            if xdict['op']:
+                if xdict['op'].isspace():
+                    xdict['op'] = '*'
+                out += xdict['op']
+    else:  # pragma: debug
+        print(repr(orig_str), type(orig_str))
+        m = re.search(r'(?:%s)+' % regex, orig_str.strip())
+        if m:
+            print(repr(m.group(0)), m.groupdict())
         else:
-            out.append(xdict['name'])
-    return '*'.join(out)
+            print('no match')
+        for m in re.finditer(regex, orig_str.strip()):
+            print(m.group(0), m.groupdict())
+        raise Exception("Could not standardize units: %s" % repr(orig_str))
+    return out
 
 
-def has_units(obj):
+def has_units(obj, check_dimensionless=False):
     r"""Determine if a Python object has associated units.
 
     Args:
         obj (object): Object to be tested for units.
+        check_dimensionless (bool, optional): If True, an object with
+            dimensionless units will return True.
 
     Returns:
         bool: True if the object has units, False otherwise.
 
     """
-    out = hasattr(obj, 'units')
-    if out and (obj.units == as_unit('dimensionless')):
+    out = isinstance(obj, (_unit_quantity, _unit_array))
+    # out = hasattr(obj, 'units')
+    if ((out and (obj.units == as_unit('dimensionless'))
+         and (not check_dimensionless))):
         out = False
     return out
 
@@ -164,7 +212,7 @@ def get_data(obj):
         np.ndarray: Numpy array representation of the underlying data.
 
     """
-    if has_units(obj):
+    if has_units(obj, check_dimensionless=True):
         out = obj.to_ndarray()
         if out.ndim == 0:
             out = out.reshape((1, ))[0]
@@ -191,6 +239,7 @@ def add_units(arr, unit_str, dtype=None):
     unit_str = tools.bytes2str(unit_str)
     if is_null_unit(unit_str):
         return arr
+    unit_str = convert_unit_string(unit_str)
     if has_units(arr):
         return convert_to(arr, unit_str)
     if dtype is None:
@@ -198,12 +247,16 @@ def add_units(arr, unit_str, dtype=None):
             dtype = arr.dtype
         else:
             dtype = np.array([arr]).dtype
-    if isinstance(arr, np.ndarray) and (arr.ndim > 0):
-        out = unyt.unyt_array(arr, unit_str, dtype=dtype,
-                              registry=ureg)
-    else:
-        out = unyt.unyt_quantity(arr, unit_str, dtype=dtype,
-                                 registry=ureg)
+    try:
+        if isinstance(arr, np.ndarray) and (arr.ndim > 0):
+            out = unyt.unyt_array(arr, unit_str, dtype=dtype,
+                                  registry=ureg)
+        else:
+            out = unyt.unyt_quantity(arr, unit_str, dtype=dtype,
+                                     registry=ureg)
+    except BaseException:
+        raise ValueError("Error parsing unit: %s, type(%s)."
+                         % (repr(unit_str), type(unit_str)))
     return out
 
 
